#include "ktexteditor.h"
#include "themeController.h"

#include <QScrollArea>
#include <QBoxLayout>
#include <QTextEdit>
#include <QPainter>
#include <QScrollBar>
#include <QEvent>
#include <QDebug>

namespace kdk {

class KTextEditorPrivate : public QObject, public ThemeController
{
    Q_OBJECT
    Q_DECLARE_PUBLIC(KTextEditor)
public:
    KTextEditorPrivate(KTextEditor* parent );
    virtual ~KTextEditorPrivate();

private:
    KTextEditor *q_ptr;
    QVBoxLayout* m_pVBoxLayout;
    QScrollArea* m_pScrollArea;
    QTextEdit* m_pTextEdit;
    bool m_focus;
};

KTextEditor::KTextEditor(QWidget *parent)
    :QWidget(parent)
    ,d_ptr(new KTextEditorPrivate(this))
{
    Q_D(KTextEditor);
    this->setFocusPolicy(Qt::ClickFocus);
    installEventFilter(this);
    d->m_pTextEdit->installEventFilter(this);
    d->m_pScrollArea->installEventFilter(this);
}

QTextEdit *KTextEditor::textEdit()
{
    Q_D(KTextEditor);
    return d->m_pTextEdit;
}

void KTextEditor::paintEvent(QPaintEvent *event)
{
    Q_D(KTextEditor);
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.fillRect(this->rect(), palette().window().color());
    QPen pen;
    pen.setWidth(2);
    qDebug()<< d->m_pTextEdit->hasFocus()<<d->m_pScrollArea->hasFocus() << hasFocus() <<d->m_focus;
    if(d->m_focus)
    {
        pen.setColor(palette().highlight().color());
        painter.setPen(pen);
    }
    else
    {
        pen.setColor(palette().placeholderText().color());
        painter.setPen(pen);
    }

    painter.setBrush(palette().base().color());
    QRect rect = this->rect().adjusted(1, 1, -1, -1);
    painter.drawRoundedRect(rect, 8, 8);
}

bool KTextEditor::eventFilter(QObject *watched, QEvent *event)
{
    Q_D(KTextEditor);
    if(watched == this)
    {
        switch (event->type())
        {
        qDebug()<<event->type();
        case QEvent::FocusIn:
            qDebug()<<"focus in";
            d->m_focus = true;
            update();
            break;
        case QEvent::FocusOut:
            qDebug()<<"focus out";
            d->m_focus = false;
            update();
            break;
        default:
            break;
        }
        return QWidget::eventFilter(watched,event);
    }
    else
    {
        switch (event->type())
        {
        qDebug()<<event->type();
        case QEvent::FocusIn:
            qDebug()<<"focus in";
            d->m_focus = true;
            update();
            break;
        case QEvent::FocusOut:
            qDebug()<<"focus out";
            d->m_focus = false;
            update();
            break;
        default:
            break;
        }
        return false;
    }
}

KTextEditorPrivate::KTextEditorPrivate(KTextEditor *parent)
    :q_ptr(parent)
{
    Q_Q(KTextEditor);

    m_pVBoxLayout = new QVBoxLayout();
    m_pScrollArea = new QScrollArea();
    m_pTextEdit = new QTextEdit();
    m_focus = false;

    q->setContentsMargins(8,12,0,13);//上下左右边距为20 ，因控件自带边距调整
    m_pScrollArea->setWidget(m_pTextEdit);
    m_pScrollArea->horizontalScrollBar()->setVisible(false);
    m_pScrollArea->setWidgetResizable(true);
    m_pScrollArea->setFocusPolicy(Qt::ClickFocus);

    m_pTextEdit->setFrameShape(QFrame::NoFrame);
    m_pScrollArea->setFrameShape(QFrame::NoFrame);


    m_pVBoxLayout->addWidget(m_pScrollArea);

    q->setLayout(m_pVBoxLayout);
}

KTextEditorPrivate::~KTextEditorPrivate()
{

}

}

#include "ktexteditor.moc"
#include "moc_ktexteditor.cpp"
